@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -44,9 +44,9 @@ class swap ~kind (source : source) =
         buffer.(2) <- tmp)
       else
         for i = offset to AFrame.position buf - 1 do
-          let tmp = buffer.(0).{i} in
-          buffer.(0).{i} <- buffer.(1).{i};
-          buffer.(1).{i} <- tmp
+          let tmp = buffer.(0).(i) in
+          buffer.(0).(i) <- buffer.(1).(i);
+          buffer.(1).(i) <- tmp
         done
   end
 
