@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -459,7 +459,7 @@ let mk_buffer ~ctype generator =
           | Some c -> c
       in
       fun ?pts ~samplerate data ->
-        let data = resampler ~samplerate data in
+        let data, _, _ = resampler ~samplerate data 0 (Audio.length data) in
         let data = (get_channel_converter ()) data in
         let len = Audio.length data in
         let data = Frame_content.Audio.lift_data data in
