@@ -1,7 +1,7 @@
 (*****************************************************************************
 
    Liquidsoap, a programmable audio stream generator.
-   Copyright 2003-2021 Savonet team
+   Copyright 2003-2022 Savonet team
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -693,6 +693,7 @@ let mk_decoder ?audio ?video ~decode_first_metadata ~target_position container =
     f ()
 
 let mk_streams ~ctype container =
+  let stream_idx = Ffmpeg_content_base.new_stream_idx () in
   let audio =
     try
       match ctype.Frame.audio with
@@ -700,10 +701,13 @@ let mk_streams ~ctype container =
         | f when Ffmpeg_copy_content.Audio.is_format f ->
             Some
               (`Packet
-                (Ffmpeg_copy_decoder.mk_audio_decoder ~format:f container))
+                (Ffmpeg_copy_decoder.mk_audio_decoder ~stream_idx ~format:f
+                   container))
         | f when Ffmpeg_raw_content.Audio.is_format f ->
             Some
-              (`Frame (Ffmpeg_raw_decoder.mk_audio_decoder ~format:f container))
+              (`Frame
+                (Ffmpeg_raw_decoder.mk_audio_decoder ~stream_idx ~format:f
+                   container))
         | f ->
             let channels = Frame_content.Audio.channels_of_format f in
             Some
@@ -718,10 +722,13 @@ let mk_streams ~ctype container =
         | f when Ffmpeg_copy_content.Video.is_format f ->
             Some
               (`Packet
-                (Ffmpeg_copy_decoder.mk_video_decoder ~format:f container))
+                (Ffmpeg_copy_decoder.mk_video_decoder ~stream_idx ~format:f
+                   container))
         | f when Ffmpeg_raw_content.Video.is_format f ->
             Some
-              (`Frame (Ffmpeg_raw_decoder.mk_video_decoder ~format:f container))
+              (`Frame
+                (Ffmpeg_raw_decoder.mk_video_decoder ~stream_idx ~format:f
+                   container))
         | _ ->
             Some (`Frame (Ffmpeg_internal_decoder.mk_video_decoder container))
     with Avutil.Error _ -> None
