@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -81,7 +81,7 @@ let encoder id ext =
         log#important "Error: %s" (Printexc.to_string e);
         restart_decision ()
   in
-  let log = log#important "%s" in
+  let log s = log#important "%s" s in
   let on_stdout =
     Tutils.mutexify mutex (fun puller ->
         begin
@@ -115,20 +115,15 @@ let encoder id ext =
           ~converter frame start len
       else (
         let b = AFrame.pcm frame in
+        let start = Frame.audio_of_main start in
         let len = Frame.audio_of_main len in
         (* Resample if needed. *)
         let b, start, len =
-          if ratio = 1. then (b, start, len)
-          else (
-            let b =
-              Audio_converter.Samplerate.resample converter ratio
-                (Audio.sub b start len)
-            in
-            (b, 0, Audio.length b))
+          Audio_converter.Samplerate.resample converter ratio b start len
         in
         let slen = 2 * len * Array.length b in
         let sbuf = Bytes.create slen in
-        Audio.S16LE.of_audio (Audio.sub b start len) sbuf 0;
+        Audio.S16LE.of_audio b start sbuf 0 len;
         Strings.unsafe_of_bytes sbuf)
     in
     Tutils.mutexify mutex
