@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 open Mm
 
 module InternalResampler =
-  Swresample.Make (Swresample.Frame) (Swresample.FltPlanarBigArray)
+  Swresample.Make (Swresample.Frame) (Swresample.PlanarFloatArray)
 
 module InternalScaler = Swscale.Make (Swscale.Frame) (Swscale.BigArray)
 
@@ -524,9 +524,7 @@ let mk_encoder mode =
       in
       new Producer_consumer.producer
       (* We are expecting real-rate with a couple of hickups.. *)
-        ~check_self_sync:false
-        ~consumers_val:[Lang.source (consumer :> Source.source)]
-        ~name:(id ^ ".producer")
+        ~check_self_sync:false ~consumers:[consumer] ~name:(id ^ ".producer")
         ~kind:(Source.Kind.of_kind return_kind)
         generator)
 
