@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 open Mm
 
 module InternalResampler =
-  Swresample.Make (Swresample.FltPlanarBigArray) (Swresample.Frame)
+  Swresample.Make (Swresample.PlanarFloatArray) (Swresample.Frame)
 
 module InternalScaler = Swscale.Make (Swscale.BigArray) (Swscale.Frame)
 
@@ -160,8 +160,10 @@ let encode_audio_frame ~kind_t ~mode ~opts ?codec ~format generator =
 
   function
   | `Frame frame ->
-      let pcm = Audio.sub (AFrame.pcm frame) 0 (AFrame.position frame) in
-      let frame = InternalResampler.convert resampler pcm in
+      let frame =
+        InternalResampler.convert ~length:(AFrame.position frame) resampler
+          (AFrame.pcm frame)
+      in
       encode_ffmpeg_frame frame
   | `Flush -> encode_frame `Flush
 
@@ -625,8 +627,7 @@ let mk_encoder mode =
       in
       new Producer_consumer.producer
       (* We are expecting real-rate with a couple of hickups.. *)
-        ~check_self_sync:false
-        ~consumers_val:[Lang.source (consumer :> Source.source)]
+        ~check_self_sync:false ~consumers:[consumer]
         ~kind:(Source.Kind.of_kind return_kind)
         ~name:(id ^ ".producer") generator)
 
