@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,40 @@
 
  *****************************************************************************)
 
+(** FFmpeg filter graphs initialization is pretty tricky. Things to consider:
+  * - FFmpeg filters are using a push paradigm, pushing from the sources down
+  *   to the outputs
+  * - Liquidsoap uses a pull paradigm, pulling data from the outputs.
+  * - FFmpeg filters inputs need to know the exact content of the data sent to
+  *   them before being initialized, which is only known in the worst case
+  *   when receiving the first frame.
+  *
+  * Therefore, the intended implementation is to:
+  * -> Consider ffmpeg filter graphs as a single operator with N inputs
+  *    and M outputs (audio/video) with inputs being any source converted to
+  *    a ffmpeg graph input, even if not used, for simplification.
+  * -> The outputs are placed in their clock which controls a child clock
+  *    containing the inputs.
+  * -> The graph initialization is suspended until its initialization conditions
+  *    are met.
+  * -> When all the inputs have been initialized and are ready (call to `#is_ready`),
+  *    the graph is considered ready and its output can start requesting
+  *    content. This is captured by the call to `is_ready` below.
+  * -> When requesting content, the outputs can tick the input clock as many time
+  *    as needed to generate output data. We expect more or less real time with
+  *    perhaps an accordion pattern between input and output so we do not
+  *    look at latency control like we do for crossfades.
+  * -> When receiving its first frame, the liquidsoap input will then initialize the
+  *    corresponding ffmpeg graph input with full format info.
+  * -> When all inputs have been initialized, which is know by checking if all of 
+  *    the graph's lazy values for input pads have been forced (executed), 
+  *    the whole graph is initialized, outputs connected and data can start to flow!
+  *    This is captured by the call to `initialized` below.
+  *
+  * This is contingent to the inputs being checked for `#is_ready` and the
+  * outputs only pulling when _all_ inputs are available, to avoid running
+  * into endless pulling loops. *)
+
 let () =
   Lang.add_module "ffmpeg.filter";
   Lang.add_module "ffmpeg.filter.audio";
@@ -38,19 +72,42 @@ type outputs =
 
 type graph = {
   mutable config : Avfilter.config option;
+  mutable failed : bool;
   input_inits : (unit -> bool) Queue.t;
+  graph_inputs : Source.source Queue.t;
+  graph_outputs : Source.source Queue.t;
   init : unit Lazy.t Queue.t;
   entries : (inputs, outputs) Avfilter.io;
-  clocks : Source.clock_variable Queue.t;
 }
 
 let init_graph graph =
-  if Queue.fold (fun b v -> b && v ()) true graph.input_inits then
-    Queue.iter Lazy.force graph.init
+  if Queue.fold (fun b v -> b && v ()) true graph.input_inits then (
+    try Queue.iter Lazy.force graph.init
+    with exn ->
+      let bt = Printexc.get_raw_backtrace () in
+      graph.failed <- true;
+      Printexc.raise_with_backtrace exn bt)
 
-let initiated graph =
+let initialized graph =
   Queue.fold (fun cur q -> cur && Lazy.is_val q) true graph.init
 
+let is_ready graph =
+  (not graph.failed)
+  && Queue.fold (fun cur s -> cur && s#is_ready) true graph.graph_inputs
+
+let pull graph =
+  (Clock.get (Queue.peek graph.graph_inputs)#clock)#end_tick;
+  Queue.iter (fun s -> s#after_output) graph.graph_inputs
+
+let self_sync_type graph =
+  Lazy.from_fun (fun () ->
+      Lazy.force
+        (Utils.self_sync_type
+           (Queue.fold (fun cur s -> s :: cur) [] graph.graph_inputs)))
+
+let self_sync graph =
+  Queue.fold (fun cur s -> cur || snd s#self_sync) false graph.graph_inputs
+
 module Graph = Lang.MkAbstract (struct
   type content = graph
 
@@ -300,7 +357,7 @@ let apply_filter ~args_parser ~filter ~sources_t p =
               let flags = if fast then [`Fast] else [] in
               let cmd = Lang.to_string (Lang.assoc "" 1 p) in
               let arg = Lang.to_string (Lang.assoc "" 2 p) in
-              if initiated graph then (
+              if initialized graph then (
                 try
                   Lang.string (Avfilter.process_command ~flags ~cmd ~arg filter)
                 with exn ->
@@ -515,19 +572,10 @@ let () =
     { Frame.audio = Frame.none; video = raw_video_format; midi = Frame.none }
   in
   let audio_t =
-    Lang.(source_t ~methods:true (kind_type_of_kind_format audio_frame))
+    Lang.(source_t ~methods:false (kind_type_of_kind_format audio_frame))
   in
   let video_t =
-    Lang.(source_t ~methods:true (kind_type_of_kind_format video_frame))
-  in
-
-  let output_base_proto =
-    [
-      ( "buffer",
-        Lang.float_t,
-        Some (Lang.float 0.1),
-        Some "Duration of the pre-buffered data." );
-    ]
+    Lang.(source_t ~methods:false (kind_type_of_kind_format video_frame))
   in
 
   Lang.add_builtin ~category:`Filter "ffmpeg.filter.audio.input"
@@ -574,7 +622,7 @@ let () =
           | Source.Kind.Conflict (a, b) ->
               raise (Error.Kind_conflict (pos, a, b))
       in
-      Queue.add s#clock graph.clocks;
+      Queue.add (s :> Source.source) graph.graph_inputs;
 
       let args = ref None in
 
@@ -603,15 +651,14 @@ let () =
   let return_t = Lang.kind_type_of_kind_format return_kind in
   Lang.add_operator "ffmpeg.filter.audio.output" ~category:`Audio
     ~descr:"Return an audio source from a filter's output" ~return_t
-    (output_base_proto
-    @ [
-        ( "pass_metadata",
-          Lang.bool_t,
-          Some (Lang.bool true),
-          Some "Pass ffmpeg stream metadata to liquidsoap" );
-        ("", Graph.t, None, None);
-        ("", Audio.t, None, None);
-      ])
+    [
+      ( "pass_metadata",
+        Lang.bool_t,
+        Some (Lang.bool true),
+        Some "Pass ffmpeg stream metadata to liquidsoap" );
+      ("", Graph.t, None, None);
+      ("", Audio.t, None, None);
+    ]
     (fun p ->
       let pass_metadata = Lang.to_bool (List.assoc "pass_metadata" p) in
       let graph_v = Lang.assoc "" 1 p in
@@ -627,9 +674,14 @@ let () =
               midi = none;
             }
       in
-      let bufferize = Lang.to_float (List.assoc "buffer" p) in
-      let s = new Ffmpeg_filter_io.audio_input ~pass_metadata ~bufferize kind in
-      Queue.add s#clock graph.clocks;
+      let s =
+        new Ffmpeg_filter_io.audio_input
+          ~pull:(fun () -> pull graph)
+          ~is_ready:(fun () -> is_ready graph)
+          ~self_sync:(fun () -> self_sync graph)
+          ~self_sync_type:(self_sync_type graph) ~pass_metadata kind
+      in
+      Queue.add (s :> Source.source) graph.graph_outputs;
 
       let pad = Audio.of_value (Lang.assoc "" 2 p) in
       Queue.add
@@ -691,7 +743,7 @@ let () =
           | Source.Kind.Conflict (a, b) ->
               raise (Error.Kind_conflict (pos, a, b))
       in
-      Queue.add s#clock graph.clocks;
+      Queue.add (s :> Source.source) graph.graph_inputs;
 
       let args = ref None in
 
@@ -719,19 +771,18 @@ let () =
   let return_t = Lang.kind_type_of_kind_format return_kind in
   Lang.add_operator "ffmpeg.filter.video.output" ~category:`Video
     ~descr:"Return a video source from a filter's output" ~return_t
-    (output_base_proto
-    @ [
-        ( "pass_metadata",
-          Lang.bool_t,
-          Some (Lang.bool false),
-          Some "Pass ffmpeg stream metadata to liquidsoap" );
-        ( "fps",
-          Lang.nullable_t Lang.int_t,
-          Some Lang.null,
-          Some "Output frame per seconds. Defaults to global value" );
-        ("", Graph.t, None, None);
-        ("", Video.t, None, None);
-      ])
+    [
+      ( "pass_metadata",
+        Lang.bool_t,
+        Some (Lang.bool false),
+        Some "Pass ffmpeg stream metadata to liquidsoap" );
+      ( "fps",
+        Lang.nullable_t Lang.int_t,
+        Some Lang.null,
+        Some "Output frame per seconds. Defaults to global value" );
+      ("", Graph.t, None, None);
+      ("", Video.t, None, None);
+    ]
     (fun p ->
       let pass_metadata = Lang.to_bool (List.assoc "pass_metadata" p) in
       let graph_v = Lang.assoc "" 1 p in
@@ -751,11 +802,14 @@ let () =
               midi = none;
             }
       in
-      let bufferize = Lang.to_float (List.assoc "buffer" p) in
       let s =
-        new Ffmpeg_filter_io.video_input ~pass_metadata ~bufferize ~fps kind
+        new Ffmpeg_filter_io.video_input
+          ~pull:(fun () -> pull graph)
+          ~is_ready:(fun () -> is_ready graph)
+          ~self_sync:(fun () -> self_sync graph)
+          ~self_sync_type:(self_sync_type graph) ~pass_metadata ~fps kind
       in
-      Queue.add s#clock graph.clocks;
+      Queue.add (s :> Source.source) graph.graph_outputs;
 
       Queue.add
         (lazy
@@ -786,6 +840,9 @@ let () =
 
       (s :> Source.source))
 
+let unify_clocks ~clock sources =
+  Queue.iter (fun s -> Clock.unify clock s#clock) sources
+
 let () =
   let univ_t = Lang.univ_t () in
   Lang.add_builtin "ffmpeg.filter.create" ~category:`Filter
@@ -799,9 +856,11 @@ let () =
         Avfilter.
           {
             config = Some config;
+            failed = false;
             input_inits = Queue.create ();
+            graph_inputs = Queue.create ();
+            graph_outputs = Queue.create ();
             init = Queue.create ();
-            clocks = Queue.create ();
             entries =
               {
                 inputs =
@@ -812,8 +871,14 @@ let () =
           }
       in
       let ret = Lang.apply fn [("", Graph.to_value graph)] in
-      let first = Queue.take graph.clocks in
-      Queue.iter (Clock.unify first) graph.clocks;
+      let input_clock =
+        Clock.create_known (new Clock.clock ~start:false "ffmpeg.filter")
+      in
+      unify_clocks ~clock:input_clock graph.graph_inputs;
+      let output_clock =
+        Clock.create_unknown ~sources:[] ~sub_clocks:[input_clock]
+      in
+      unify_clocks ~clock:output_clock graph.graph_outputs;
       Queue.add
         (lazy
           (log#info "Initializing graph";
