@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -565,29 +565,22 @@ module MkAbstract (Def : AbstractDef) = struct
   type G.t += Value of Def.content
 
   let () =
-    G.register (function
-      | Value v ->
-          let compare = function
-            | Value v' -> Def.compare v v'
-            | _ -> assert false
-          in
-          Some
-            {
-              G.descr = (fun () -> Def.descr v);
-              to_json =
-                (fun ~compact ~json5 () -> Def.to_json ~compact ~json5 v);
-              compare;
-              typ = Type;
-            }
-      | _ -> None);
-
+    let to_ground = function Value v -> v | _ -> assert false in
+    let compare v v' = Def.compare (to_ground v) (to_ground v') in
+    let to_json ~compact ~json5 v = Def.to_json ~compact ~json5 (to_ground v) in
+    let descr v = Def.descr (to_ground v) in
+    G.register
+      (function Value _ -> true | _ -> false)
+      { G.descr; to_json; compare; typ = Type };
     Type.register_ground_printer (function Type -> Some Def.name | _ -> None)
 
   let t = ground_t Type
   let to_value c = mk (Term.Value.Ground (Value c))
 
   let of_value t =
-    match t.value with Term.Value.Ground (Value c) -> c | _ -> assert false
+    match (demeth t).value with
+      | Term.Value.Ground (Value c) -> c
+      | _ -> assert false
 
   let is_value t =
     match t.value with Term.Value.Ground (Value _) -> true | _ -> false
@@ -608,7 +601,7 @@ let source_methods =
        initialized.",
       fun s -> val_fun [] (fun _ -> bool s#is_ready) );
     ( "last_metadata",
-      ([], fun_t [(false, "", fun_t [] (nullable_t metadata_t))] unit_t),
+      ([], fun_t [] (nullable_t metadata_t)),
       "Return the last metadata from the source.",
       fun s ->
         val_fun [] (fun _ ->
@@ -745,14 +738,6 @@ let source_methods =
       ([], bool_t),
       "Indicate if a source may fail, i.e. may not be ready to stream.",
       fun s -> bool (s#stype = Source.Fallible) );
-    ( "shutdown",
-      ([], fun_t [] unit_t),
-      "Deactivate a source.",
-      fun s ->
-        val_fun [] (fun _ ->
-            (Clock.get s#clock)#detach (fun (s' : Source.active_source) ->
-                (s' :> Source.source) = (s :> Source.source));
-            unit) );
     ( "time",
       ([], fun_t [] float_t),
       "Get a source's time, based on its assigned clock.",
