@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -612,12 +612,24 @@ exception Unsatisfied_constraint of constr * t
 
 (** Check that [a] (a dereferenced type variable) does not occur in [b],
   * and prepare the instantiation [a<-b] by adjusting the levels. *)
-let rec occur_check a b =
+let rec constr_check a (_, x) =
+  occur_check a x;
+  a
+
+and arrow_check a (_, _, t) =
+  occur_check a t;
+  a
+
+and tuple_check a v =
+  occur_check a v;
+  a
+
+and occur_check a b =
   let b = deref b in
   if a == b then raise (Occur_check (a, b));
   match b.descr with
-    | Constr c -> List.iter (fun (_, x) -> occur_check a x) c.params
-    | Tuple l -> List.iter (occur_check a) l
+    | Constr c -> ignore (List.fold_left constr_check a c.params)
+    | Tuple l -> ignore (List.fold_left tuple_check a l)
     | Getter t -> occur_check a t
     | List t -> occur_check a t
     | Nullable t -> occur_check a t
@@ -626,7 +638,7 @@ let rec occur_check a b =
         occur_check a t;
         occur_check a u
     | Arrow (p, t) ->
-        List.iter (fun (_, _, t) -> occur_check a t) p;
+        ignore (List.fold_left arrow_check a p);
         occur_check a t
     | EVar _ ->
         (* In normal type inference level -1 should never arise.
