@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -40,9 +40,9 @@ class compand ~kind (source : source) mu =
         let b_c = b.(c) in
         for i = offset to AFrame.position buf - 1 do
           (* Cf. http://en.wikipedia.org/wiki/Mu-law *)
-          let sign = if b_c.{i} < 0. then -1. else 1. in
-          b_c.{i} <-
-            sign *. log (1. +. (mu *. abs_float b_c.{i})) /. log (1. +. mu)
+          let sign = if b_c.(i) < 0. then -1. else 1. in
+          b_c.(i) <-
+            sign *. log (1. +. (mu *. abs_float b_c.(i))) /. log (1. +. mu)
         done
       done
   end
