@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -81,9 +81,9 @@ class normalize ~kind ~track_sensitive (source : source) (* RMS target. *) rmst
       for i = offset to AFrame.position buf - 1 do
         for c = 0 to self#audio_channels - 1 do
           let bc = b.(c) in
-          let x = bc.{i} in
+          let x = bc.(i) in
           rms <- rms +. (x *. x);
-          bc.{i} <-
+          bc.(i) <-
             x
             *. ((float rmsc *. vold) +. (float (rmsi - rmsc) *. v))
             /. float rmsi
