@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -44,8 +44,8 @@ class pan ~kind (source : source) phi phi_0 =
       let gain_left = (tan phi_0 +. tan phi) /. 2. in
       let gain_right = (tan phi_0 -. tan phi) /. 2. in
       let len = AFrame.position buf - offset in
-      Audio.Mono.amplify gain_left (Audio.Mono.sub buffer.(0) offset len);
-      Audio.Mono.amplify gain_right (Audio.Mono.sub buffer.(1) offset len)
+      Audio.Mono.amplify gain_left buffer.(0) offset len;
+      Audio.Mono.amplify gain_right buffer.(1) offset len
   end
 
 let () =
