@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
  *****************************************************************************)
 
-open Mm
 open Source
 module Generator = Generator.From_frames
 
@@ -71,10 +70,12 @@ class resample ~kind ~ratio source_val =
       let ratio = ratio () in
       let content, len =
         let content = AFrame.pcm frame in
-        let content = Audio.sub content 0 (AFrame.position frame) in
         let converter = Option.get converter in
-        let pcm = Audio_converter.Samplerate.resample converter ratio content in
-        let len = Audio.length pcm in
+        let pcm, ofs, len =
+          Audio_converter.Samplerate.resample converter ratio content 0
+            (AFrame.position frame)
+        in
+        assert (ofs = 0);
         ( {
             Frame.audio = Frame_content.Audio.lift_data pcm;
             video = Frame_content.None.data;
@@ -90,12 +91,13 @@ class resample ~kind ~ratio source_val =
       if Frame.is_partial frame then Generator.add_break generator
 
     method private get_frame frame =
+      consumer#set_output_enabled true;
       while
         Generator.length generator < Lazy.force Frame.size && source#is_ready
       do
         self#child_tick
       done;
-      needs_tick <- false;
+      consumer#set_output_enabled false;
       Generator.fill generator frame
   end
 
