@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
  *****************************************************************************)
 
-open Mm
 open Source
 
 class bpm ~kind (source : source) =
@@ -48,8 +47,7 @@ class bpm ~kind (source : source) =
       source#get buf;
       let len = AFrame.position buf - offset in
       let buf = AFrame.pcm buf in
-      let ibuf = Audio.interleave (Audio.sub buf offset len) in
-      Soundtouch.BPM.put_samples_ba bpm ibuf
+      Soundtouch.BPM.put_samples_ni bpm buf offset len
 
     method bpm =
       match bpm with Some bpm -> Soundtouch.BPM.get_bpm bpm | None -> 0.
