@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -404,7 +404,9 @@ class virtual operator ?(name = "src") ?audio_in ?video_in ?midi_in out_kind
       Gc.finalise (fun s -> source_log#info "Garbage collected %s." s#id) self
 
     val mutex = Mutex.create ()
-    method mutexify : 'a 'b. ('a -> 'b) -> 'a -> 'b = Tutils.mutexify mutex
+
+    method private mutexify : 'a 'b. ('a -> 'b) -> 'a -> 'b =
+      Tutils.mutexify mutex
 
     (** Is the source infallible, i.e. is it always guaranteed that there
     * will be always be a next track immediately available. *)
@@ -542,7 +544,6 @@ class virtual operator ?(name = "src") ?audio_in ?video_in ?midi_in out_kind
         source_log#info "Source %s gets up with content kind: %s." id
           (Kind.to_string self#kind);
         self#wake_up activation);
-      Server.register_op self#id name;
       if dynamic then dynamic_activations <- activation :: dynamic_activations
       else static_activations <- activation :: static_activations;
       self#update_caching_mode;
@@ -579,7 +580,7 @@ class virtual operator ?(name = "src") ?audio_in ?video_in ?midi_in out_kind
       else static_activations <- remove [] static_activations;
       self#update_caching_mode;
       if static_activations = [] && dynamic_activations = [] then (
-        source_log#debug "Source %s gets down." id;
+        source_log#info "Source %s gets down." id;
         self#mutexify
           (fun () ->
             List.iter (fun fn -> try fn () with _ -> ()) on_shutdown;
@@ -777,7 +778,7 @@ class virtual operator ?(name = "src") ?audio_in ?video_in ?midi_in out_kind
       self#iter_watchers (fun w -> w.after_output ())
 
     (* Reset the cache frame *)
-    method advance = Frame.advance self#memo
+    method advance = Frame.clear self#memo
 
     (** Utils. *)
 
