@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ class gen ~kind ~seek name g freq duration ampl =
       let buf = AFrame.pcm frame in
       g#set_frequency (freq ());
       g#set_volume (ampl ());
-      g#fill (Audio.sub buf off len)
+      g#fill buf off len
   end
 
 let add name g =
