@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -47,10 +47,9 @@ class blank ~kind duration =
       let video_pos = Frame.video_of_main position in
       (* Audio *)
       (try
-         Audio.clear
-           (Audio.sub (AFrame.pcm ab)
-              (Frame.audio_of_main position)
-              (Frame.audio_of_main length))
+         Audio.clear (AFrame.pcm ab)
+           (Frame.audio_of_main position)
+           (Frame.audio_of_main length)
        with Frame_content.Invalid -> ());
 
       (* Video *)
