@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ class noise ~kind duration =
          let off = Frame.audio_of_main off in
          let len = Frame.audio_of_main len in
          let b = AFrame.pcm frame in
-         Audio.Generator.white_noise (Audio.sub b off len)
+         Audio.Generator.white_noise b off len
        with Frame_content.Invalid -> ());
       try
         let off = Frame.video_of_main off in
