@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ let pcm b = Frame_content.Audio.get_data (content b)
 let to_s16le b =
   let fpcm = pcm b in
   assert (Audio.channels fpcm = 2);
-  Audio.S16LE.make fpcm
+  Audio.S16LE.make fpcm 0 (Audio.length fpcm)
 
 let duration () = Lazy.force duration
 let size () = sot (Lazy.force size)
@@ -43,7 +43,6 @@ let breaks t = List.map sot (breaks t)
 let add_break t i = add_break t (tos i)
 let set_breaks t l = set_breaks t (List.map tos l)
 let is_partial = is_partial
-let advance = advance
 let clear = clear
 
 exception No_metadata
@@ -61,10 +60,7 @@ let set_all_metadata t l =
 
 let free_metadata = free_metadata
 let free_all_metadata = free_all_metadata
-let blankify b off len = Audio.clear (Audio.sub (pcm b) off len)
-let multiply b off len c = Audio.amplify c (Audio.sub (pcm b) off len)
-
-let add b1 off1 b2 off2 len =
-  Audio.add (Audio.sub (pcm b1) off1 len) (Audio.sub (pcm b2) off2 len)
-
-let rms b off len = Audio.Analyze.rms (Audio.sub (pcm b) off len)
+let blankify b off len = Audio.clear (pcm b) off len
+let multiply b off len c = Audio.amplify c (pcm b) off len
+let add b1 off1 b2 off2 len = Audio.add (pcm b1) off1 (pcm b2) off2 len
+let rms b off len = Audio.Analyze.rms (pcm b) off len
