@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@ class synth ~kind (synth : Synth.synth) (source : source) chan volume =
         let b = AFrame.pcm buf in
         let position = AFrame.position buf in
         let len = position - offset in
-        synth#play evs offset (Audio.sub b offset len))
+        synth#play evs offset b offset len)
   end
 
 let register obj name descr =
