@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,20 +32,21 @@ module Liq_http = struct
     | e -> Http (Printexc.to_string e)
 
   (* This in unused for now.. *)
-  let default_timeout = ref 5.
+  let default_timeout = ref 50.
 
   let request ?timeout ?headers ?(port = 80) ~host ~url ~request () =
     try
       let timeout =
-        int_of_float
-          (match timeout with None -> !default_timeout | Some t -> t)
+        match timeout with None -> !default_timeout | Some t -> t
       in
       let request = match request with Get -> `Get | Post s -> `Post s in
       let url = Printf.sprintf "http://%s:%d%s" host port url in
       let data = Buffer.create 1024 in
       let x, code, y, _ =
         Liqcurl.http_request ?headers ~follow_redirect:true
-          ~on_body_data:(Buffer.add_string data) ~timeout ~url ~request ()
+          ~on_body_data:(Buffer.add_string data)
+          ~timeout:(int_of_float (timeout *. 1000.))
+          ~url ~request ()
       in
       if code <> 200 then
         raise (Http (Printf.sprintf "Http request failed: %s %i %s" x code y));
@@ -89,7 +90,7 @@ let init host =
   let submissions = Queue.create () in
   (* A mutex to manage thread concurrency *)
   let submit_m = Mutex.create () in
-  let reason = log#important "Lastfm Submission failed: %s" in
+  let reason s = log#important "Lastfm Submission failed: %s" s in
   (* Define a new task *)
   let do_submit () =
     try
