@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -41,11 +41,14 @@ class virtual base ~(on_start : unit -> unit) ~(on_stop : unit -> unit) =
 
     (* Default [reset] method. Can be overriden if necessary. *)
     method reset =
-      self#stop;
-      self#start
+      match self#state with
+        | `Started ->
+            self#stop;
+            self#start
+        | _ -> ()
 
     method transition_to (s : state) =
-      match (s, state) with
+      match (s, self#state) with
         | `Started, `Stopped | `Started, `Idle ->
             self#start;
             on_start ();
@@ -99,24 +102,26 @@ class virtual active_source ?get_clock ~name ~content_kind ~clock_safe
       if self#is_ready && AFrame.is_partial self#memo then self#get self#memo
   end
 
-let output_proto =
+let base_proto ~label =
   [
     ( "on_start",
       Lang.fun_t [] Lang.unit_t,
       Some (Lang.val_cst_fun [] Lang.unit),
-      Some "Callback executed when input starts." );
+      Some ("Callback executed when " ^ label ^ " starts.") );
     ( "on_stop",
       Lang.fun_t [] Lang.unit_t,
       Some (Lang.val_cst_fun [] Lang.unit),
-      Some "Callback executed when input stops." );
+      Some ("Callback executed when " ^ label ^ " stops.") );
     ( "start",
       Lang.bool_t,
       Some (Lang.bool true),
-      Some "Start input as soon as it is available." );
+      Some ("Start " ^ label ^ " as soon as it is available.") );
   ]
 
+let output_proto = base_proto ~label:"output"
+
 let active_source_proto ~fallible_opt ~clock_safe =
-  output_proto
+  base_proto ~label:"input"
   @ [
       ( "clock_safe",
         Lang.bool_t,
@@ -170,4 +175,13 @@ let meth :
                       });
               s#transition_to `Stopped;
               unit) );
+      ( "shutdown",
+        ([], fun_t [] unit_t),
+        "Shutdown the output or source.",
+        fun s ->
+          val_fun [] (fun _ ->
+              if Source.Clock_variables.is_known s#clock then
+                (Clock.get s#clock)#detach (fun (s' : Source.active_source) ->
+                    (s' :> Source.source) = (s :> Source.source));
+              unit) );
     ]
