@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -85,14 +85,19 @@ let scheduler_log =
     ~d:false "Log scheduler messages"
 
 let mutexify lock f x =
-  Mutex.lock lock;
+  let after =
+    try
+      Mutex.lock lock;
+      fun () -> Mutex.unlock lock
+    with Sys_error _ -> fun () -> ()
+  in
   try
     let ans = f x in
-    Mutex.unlock lock;
+    after ();
     ans
   with e ->
     let bt = Printexc.get_raw_backtrace () in
-    Mutex.unlock lock;
+    after ();
     Printexc.raise_with_backtrace e bt
 
 let finalize ~k f =
@@ -148,29 +153,13 @@ let no_problem = Condition.create ()
 
 exception Exit
 
-let error_handlers = Stack.create ()
-
-exception Error_processed
-
-let rec error_handler ~bt ~name exn =
-  try
-    Stack.iter
-      (fun handler -> if handler ~bt ~name exn then raise Error_processed)
-      error_handlers;
-    false
-  with
-    | Error_processed -> true
-    | exn ->
-        let bt = Printexc.get_backtrace () in
-        error_handler ~bt ~name exn
-
 let create ~queue f x s =
   let c = Condition.create () in
   let set = if queue then queues else all in
   mutexify lock
     (fun () ->
       let id =
-        let rec process x =
+        let process x =
           try
             f x;
             mutexify lock
@@ -181,29 +170,27 @@ let create ~queue f x s =
                 Condition.signal c)
               ()
           with e -> (
+            let raw_bt = Printexc.get_raw_backtrace () in
             let bt = Printexc.get_backtrace () in
             try
               match e with
                 | Exit -> log#info "Thread %S exited." s
                 | Failure e as exn ->
-                    let bt = Printexc.get_raw_backtrace () in
                     log#important "Thread %S failed: %s!" s e;
-                    Printexc.raise_with_backtrace exn bt
-                | e when queue && error_handler ~bt ~name:s e -> process x
+                    Printexc.raise_with_backtrace exn raw_bt
                 | e when queue ->
                     log#severe "Queue %s crashed with exception %s\n%s" s
                       (Printexc.to_string e) bt;
                     log#critical
                       "PANIC: Liquidsoap has crashed, exiting.,\n\
                        Please report at: savonet-users@lists.sf.net";
+                    flush_all ();
                     exit 1
                 | e ->
-                    let bt = Printexc.get_raw_backtrace () in
                     log#important "Thread %S aborts with exception %s!" s
                       (Printexc.to_string e);
-                    Printexc.raise_with_backtrace e bt
+                    Printexc.raise_with_backtrace e raw_bt
             with e ->
-              let raw_bt = Printexc.get_raw_backtrace () in
               let l = Pcre.split ~pat:"\n" bt in
               List.iter (log#info "%s") l;
               mutexify lock
