@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -56,10 +56,6 @@ type priority =
 (** task scheduler *)
 val scheduler : priority Duppy.scheduler
 
-(** Register queue error handlers. Should return [true] if 
-    the exception was handler by the given callback. *)
-val error_handlers : (bt:string -> name:string -> exn -> bool) Stack.t
-
 (** {1 Misc} *)
 
 (** Waits for [f()] to become true on condition [c].
