@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -94,7 +94,7 @@ class visu ~kind source =
           self#add_vol
             (Array.map
                (fun c ->
-                 let x = c.{i} in
+                 let x = c.(i) in
                  x *. x)
                acontent)
         done;
